@@ -233,7 +233,8 @@ int init_locking(int type, struct cmd_context *cmd)
 				return 1;
 		}
 		if (!find_config_tree_int(cmd, "locking/fallback_to_clustered_locking",
-					  DEFAULT_FALLBACK_TO_CLUSTERED_LOCKING))
+			    find_config_tree_int(cmd, "global/fallback_to_clustered_locking",
+						 DEFAULT_FALLBACK_TO_CLUSTERED_LOCKING)))
 			break;
 #endif
 
@@ -255,7 +256,8 @@ int init_locking(int type, struct cmd_context *cmd)
 
 	if ((type == 2 || type == 3) &&
 	    find_config_tree_int(cmd, "locking/fallback_to_local_locking",
-				 DEFAULT_FALLBACK_TO_LOCAL_LOCKING)) {
+	    	    find_config_tree_int(cmd, "global/fallback_to_local_locking",
+					 DEFAULT_FALLBACK_TO_LOCAL_LOCKING))) {
 		log_warn("WARNING: Falling back to local file-based locking.");
 		log_warn("Volume Groups with the clustered attribute will "
 			  "be inaccessible.");
@@ -292,6 +294,10 @@ int check_lvm1_vg_inactive(struct cmd_context *cmd, const char *vgname)
 	if (is_orphan_vg(vgname))
 		return 1;
 
+	/* LVM1 is only present in 2.4 kernels. */
+	if (strncmp(cmd->kernel_vsn, "2.4.", 4))
+		return 1;
+
 	if (dm_snprintf(path, sizeof(path), "%s/lvm/VGs/%s", cmd->proc_dir,
 			 vgname) < 0) {
 		log_error("LVM1 proc VG pathname too long for %s", vgname);
